# -*- coding: utf-8 -*-
"""RWC Classical Dataset Loader

More details are on https://staff.aist.go.jp/m.goto/RWC-MDB/rwc-mdb-c.html .

Attributes:
    METADATA_REMOTE (RemoteFileMetadata): Metadata of the remote file
    DATASET_DIR (str): The directory name for iKala dataset. Set to `'RWC-Classical'`.

"""
import csv
import librosa
import logging
import numpy as np
import os

import mirdata.utils as utils
import mirdata.download_utils as download_utils
import mirdata.jams_utils as jams_utils

METADATA_REMOTE = download_utils.RemoteFileMetadata(
    filename='rwc-c.csv',
    url='https://github.com/magdalenafuentes/metadata/archive/master.zip',
    checksum='7dbe87fedbaaa1f348625a2af1d78030',
    destination_dir=None,
)
DATASET_DIR = 'RWC-Classical'
ANNOTATIONS_REMOTE_1 = download_utils.RemoteFileMetadata(
    filename='AIST.RWC-MDB-C-2001.BEAT.zip',
    url='https://staff.aist.go.jp/m.goto/RWC-MDB/AIST-Annotation/AIST.RWC-MDB-C-2001.BEAT.zip',
    checksum='e8ee05854833cbf5eb7280663f71c29b',
    destination_dir='annotations',
)
ANNOTATIONS_REMOTE_2 = download_utils.RemoteFileMetadata(
    filename='AIST.RWC-MDB-C-2001.CHORUS.zip',
    url='https://staff.aist.go.jp/m.goto/RWC-MDB/AIST-Annotation/AIST.RWC-MDB-C-2001.CHORUS.zip',
    checksum='f77bd527510376f59f5a2eed8fd7feb3',
    destination_dir='annotations',
)


def _load_metadata(data_home):

    metadata_path = os.path.join(data_home, 'metadata-master', 'rwc-c.csv')

    if not os.path.exists(metadata_path):
        logging.info(
            'Metadata file {} not found.'.format(metadata_path)
            + 'You can download the metadata file by running download()'
        )
        return None

    with open(metadata_path, 'r') as fhandle:
        dialect = csv.Sniffer().sniff(fhandle.read(1024))
        fhandle.seek(0)
        reader = csv.reader(fhandle, dialect)
        raw_data = []
        for line in reader:
            if line[0] != 'Piece No.':
                raw_data.append(line)

    metadata_index = {}
    for line in raw_data:
        if line[0] == 'Piece No.':
            continue
        p = '00' + line[0].split('.')[1][1:]
        track_id = 'RM-C{}'.format(p[len(p) - 3 :])

        metadata_index[track_id] = {
            'piece_number': line[0],
            'suffix': line[1],
            'track_number': line[2],
            'title': line[3],
            'composer': line[4],
            'artist': line[5],
            'duration': _duration_to_sec(line[6]),
            'category': line[7],
        }

    metadata_index['data_home'] = data_home

    return metadata_index


DATA = utils.LargeData('rwc_classical_index.json', _load_metadata)


class Track(object):
    """RWC Classical Track class

    Args:
        track_id (str): Track id of the Track
        data_home (str): Local path where the dataset is stored.
            If `None`, looks for the data in the default directory, `~/mir_datasets`

    Attributes:
        track_id (str): Track id
        audio_path (str): Audio path of this Track
        piece_number (str): Piece number of this Track, [1-50]
        suffix (str): M01-M06
        track_number: CD track number of this Track
        title (str): Title of The track.
            E.g., 'Symphony no.94 in G major, Hob.I-94 `The Surprise'. 1st mvmt.'
        composer (str): Composer of this Track.
            E.g., 'Haydn, Franz Joseph'
        artist (str): E.g., 'Tokyo City Philharmonic Orchestra'
        duration_sec (float): Duration of the track in seconds
        category (str): 'Symphony', 'Concerto', 'Orchestral',
            'Solo', 'Chamber', 'Vocal', or blank.\
    """

    def __init__(self, track_id, data_home=None):
        if track_id not in DATA.index:
            raise ValueError(
                '{} is not a valid track ID in RWC-Classical'.format(track_id)
            )

        self.track_id = track_id

        if data_home is None:
            data_home = utils.get_default_dataset_path(DATASET_DIR)
        self._data_home = data_home

        self._track_paths = DATA.index[track_id]

        metadata = DATA.metadata(data_home)
        if metadata is not None and track_id in metadata:
            self._track_metadata = metadata[track_id]
        else:
            self._track_metadata = {
                'piece_number': None,
                'suffix': None,
                'track_number': None,
                'title': None,
                'composer': None,
                'artist': None,
                'duration': None,
                'category': None,
            }

        self.audio_path = os.path.join(self._data_home, self._track_paths['audio'][0])

        self.piece_number = self._track_metadata['piece_number']
        self.suffix = self._track_metadata['suffix']
        self.track_number = self._track_metadata['track_number']
        self.title = self._track_metadata['title']
        self.composer = self._track_metadata['composer']
        self.artist = self._track_metadata['artist']
        self.duration = self._track_metadata['duration']
        self.category = self._track_metadata['category']

    def __repr__(self):
        repr_string = (
            "RWC-Classical Track(track_id={}, audio_path={}, "
            + "piece_number={}, suffix={}, track_number={}, title={}, composer={}, "
            + "artist={}, duration={}, category={}"
            + "sections=SectionData('intervals', 'labels'), "
            + "beats=BeatData('beat_times', 'beat_positions'))"
        )
        return repr_string.format(
            self.track_id,
            self.audio_path,
            self.piece_number,
            self.suffix,
            self.track_number,
            self.title,
            self.composer,
            self.artist,
            self.duration,
            self.category,
        )

    @utils.cached_property
    def sections(self):
        return _load_sections(
            os.path.join(self._data_home, self._track_paths['sections'][0])
        )

    @utils.cached_property
    def beats(self):
        return _load_beats(os.path.join(self._data_home, self._track_paths['beats'][0]))

    @property
    def audio(self):
        return librosa.load(self.audio_path, sr=None, mono=True)

    def to_jams(self):
        return jams_utils.jams_converter(
            beat_data=[(self.beats, None)],
            section_data=[(self.sections, None)],
            metadata=self._track_metadata,
        )


def download(data_home=None, force_overwrite=False):

    if data_home is None:
        data_home = utils.get_default_dataset_path(DATASET_DIR)

    info_message = """
        Unfortunately the audio files of the RWC-Jazz dataset are not available
        for download. If you have the RWC-Classical dataset, place the contents into a
        folder called RWC-Classical with the following structure:
            > RWC-Classical/
                > annotations/
                > audio/rwc-c-m0i with i in [1 .. 6]
                > metadata-master/
        and copy the RWC-Classical folder to {}
    """.format(
        data_home
    )

    download_utils.downloader(
        data_home,
        zip_downloads=[METADATA_REMOTE, ANNOTATIONS_REMOTE_1, ANNOTATIONS_REMOTE_2],
        info_message=info_message,
        force_overwrite=force_overwrite,
    )


def validate(data_home=None, silence=False):
    """Validate if the stored dataset is a valid version

    Args:
        data_home (str): Local path where the dataset is stored.
            If `None`, looks for the data in the default directory, `~/mir_datasets`

    Returns:
        missing_files (list): List of file paths that are in the dataset index
            but missing locally
        invalid_checksums (list): List of file paths that file exists in the dataset
            index but has a different checksum compare to the reference checksum

    """
    if data_home is None:
        data_home = utils.get_default_dataset_path(DATASET_DIR)

    missing_files, invalid_checksums = utils.validator(
        DATA.index, data_home, silence=silence
    )
    return missing_files, invalid_checksums


def track_ids():
    """Return track ids

    Returns:
        (list): A list of track ids
    """
    return list(DATA.index.keys())


def load(data_home=None):
    """Load RWC-Classical dataset

        Args:
            data_home (str): Local path where the dataset is stored.
                If `None`, looks for the data in the default directory, `~/mir_datasets`

        Returns:
            (dict): {`track_id`: track data}

        """
    if data_home is None:
        data_home = utils.get_default_dataset_path(DATASET_DIR)

    rwc_classical_data = {}
    for key in track_ids():
        rwc_classical_data[key] = Track(key, data_home=data_home)
    return rwc_classical_data


def _load_sections(sections_path):
    if not os.path.exists(sections_path):
        return None
    begs = []  # timestamps of section beginnings
    ends = []  # timestamps of section endings
    secs = []  # section labels

    with open(sections_path, 'r') as fhandle:
        reader = csv.reader(fhandle, delimiter='\t')
        for line in reader:
            begs.append(float(line[0]) / 100.0)
            ends.append(float(line[1]) / 100.0)
            secs.append(line[2])

    return utils.SectionData(np.array([begs, ends]).T, secs)


def _position_in_bar(beat_positions, beat_times):
    """
    Mapping to beat position in bar (e.g. 1, 2, 3, 4).
    """
    # Remove -1
    _beat_positions = np.delete(beat_positions, np.where(beat_positions == -1))
    beat_times_corrected = np.delete(beat_times, np.where(beat_positions == -1))

    # Create corrected array with downbeat positions
    beat_positions_corrected = np.zeros((len(_beat_positions),))
    downbeat_positions = np.where(_beat_positions == np.max(_beat_positions))[0]
    _beat_positions[downbeat_positions] = 1
    beat_positions_corrected[downbeat_positions] = 1

    # Propagate positions
    for b in range(0, len(_beat_positions)):
        if _beat_positions[b] > _beat_positions[b - 1]:
            beat_positions_corrected[b] = beat_positions_corrected[b - 1] + 1

    if not downbeat_positions[0] == 0:
        timesig_next_bar = beat_positions_corrected[downbeat_positions[1] - 1]
        for b in range(1, downbeat_positions[0] + 1):
            beat_positions_corrected[downbeat_positions[0] - b] = (
                timesig_next_bar - b + 1
            )

    return beat_positions_corrected, beat_times_corrected


def _load_beats(beats_path):
    if not os.path.exists(beats_path):
        return None
    beat_times = []  # timestamps of beat interval beginnings
    beat_positions = []  # beat position inside the bar

    with open(beats_path, 'r') as fhandle:
        reader = csv.reader(fhandle, delimiter='\t')
        for line in reader:
            beat_times.append(float(line[0]) / 100.0)
            beat_positions.append(int(line[2]))
    beat_positions, beat_times = _position_in_bar(
        np.array(beat_positions), np.array(beat_times)
    )

    return utils.BeatData(beat_times, beat_positions.astype(int))


def _duration_to_sec(duration):
    if type(duration) == str:
        if ':' in duration:
            if len(duration.split(':')) <= 2:
                minutes, secs = duration.split(':')
            else:
                minutes, secs, _ = duration.split(
                    ':'
                )  # mistake in annotation in RM-J044
            total_secs = float(minutes) * 60 + float(secs)
            return total_secs


def _load_metadata(data_home):
    metadata_path = os.path.join(data_home, 'metadata-master', 'rwc-c.csv')

    if not os.path.exists(metadata_path):
        logging.info(
            'Metadata file {} not found.'.format(metadata_path)
            + 'You can download the metadata file by running download()'
        )
        return None

    with open(metadata_path, 'r') as fhandle:
        dialect = csv.Sniffer().sniff(fhandle.read(1024))
        fhandle.seek(0)
        reader = csv.reader(fhandle, dialect)
        raw_data = []
        for line in reader:
            if line[0] != 'Piece No.':
                raw_data.append(line)

    metadata_index = {}
    for line in raw_data:
        if line[0] == 'Piece No.':
            continue
        p = '00' + line[0].split('.')[1][1:]
        track_id = 'RM-C{}'.format(p[len(p) - 3 :])

        metadata_index[track_id] = {
            'piece_number': line[0],
            'suffix': line[1],
            'track_number': line[2],
            'title': line[3],
            'composer': line[4],
            'artist': line[5],
            'duration': _duration_to_sec(line[6]),
            'category': line[7],
        }

    metadata_index['data_home'] = data_home

    return metadata_index


def cite():
    cite_data = """
===========  MLA ===========

Goto, Masataka, et al.,
"RWC Music Database: Popular, Classical and Jazz Music Databases.",
3rd International Society for Music Information Retrieval Conference (2002)

========== Bibtex ==========

@inproceedings{goto2002rwc,
  title={RWC Music Database: Popular, Classical and Jazz Music Databases.},
  author={Goto, Masataka and Hashiguchi, Hiroki and Nishimura, Takuichi and Oka, Ryuichi},
  booktitle={3rd International Society for Music Information Retrieval Conference},
  year={2002},
  series={ISMIR},
}

"""

    print(cite_data)



# -*- coding: utf-8 -*-
"""functions for converting mirdata annotations to jams format
"""
from __future__ import absolute_import
from __future__ import division
from __future__ import print_function

import jams
import mirdata.utils as utils


def jams_converter(
    beat_data=None,
    chord_data=None,
    note_data=None,
    f0_data=None,
    section_data=None,
    multi_section_data=None,
    key_data=None,
    lyrics_data=None,
    metadata=None,
):

    """Convert annotations from a track to JAMS format.

    Parameters
    ----------
    beat_data (list or None):
        A list of tuples of (BeatData, str), where str describes the annotation (e.g. 'beats_1').
    chord_data (list or None):
        A list of tuples of (ChordData, str), where str describes the annotation.
    note_data (list or None):
        A list of tuples of (NoteData, str), where str describes the annotation.
    f0_data (list or None):
        A list of tuples of (F0Data, str), where str describes the annotation.
    section_data (list or None):
        A list of tuples of (SectionData, str), where str describes the annotation.
    multi_section_data (list or None):
        A list of tuples. Tuples in multi_section_data should contain another
        list of tuples, indicating annotations in the different levels
        e.g. ([(segments0, level0), '(segments1, level1)], annotator) and a str
        indicating the annotator
    key_data (list or None):
        A list of tuples of (KeyData, str), where str describes the annotation.
    lyrics_data (list or None):
        A list of tuples of (LyricData, str), where str describes the annotation.
    metadata (dict or None):
        A dictionary containing the track metadata.

    Returns
    -------
    jam: JAM object
        A JAM object with all the annotations.
    """

    jam = jams.JAMS()

    # metadata
    if metadata is not None:
        for key in metadata:
            if hasattr(jam.file_metadata, key):
                setattr(jam.file_metadata, key, metadata[key])
            else:
                setattr(jam.sandbox, key, metadata[key])
    # beats
    if beat_data is not None:
        if type(beat_data) != list:
            raise TypeError('beat_data should be a list of tuples')
        for beats in beat_data:
            if type(beats) != tuple:
                raise TypeError(
                    'beat_data should be a list of tuples, '
                    + 'but is a list of something else'
                )
            jam.annotations.append(beats_to_jams(beats))

    # sections
    if section_data is not None:
        if type(section_data) != list:
            raise TypeError('section_data should be a list of tuples')
        for sections in section_data:
            if type(sections) != tuple:
                raise TypeError(
                    'section_data should be a list of tuples, '
                    + 'but is a list of something else'
                )
            jam.annotations.append(sections_to_jams(sections))

    # multi-sections (sections with multiple levels)
    if multi_section_data is not None:
        if type(multi_section_data) != list:
            raise TypeError('multi_section_data should be a list of tuples')
        for sections in multi_section_data:
            if type(sections) != tuple:
                raise TypeError(
                    'multi_section_data should be a list of tuples, '
                    + 'but is a list of something else'
                )
            if (type(sections[0]) != list) or (type(sections[0][0]) != tuple):
                raise TypeError(
                    'tuples in multi_section_data should contain a '
                    + 'list of tuples, indicating annotations in the different '
                    + 'levels, e.g. ([(segments0, level0), '
                    + '(segments1, level1)], annotator)'
                )
            jam.annotations.append(multi_sections_to_jams(sections))

    # chords
    if chord_data is not None:
        if type(chord_data) != list:
            raise TypeError('chord_data should be a list of tuples')
        for chords in chord_data:
            if type(chords) != tuple:
                raise TypeError(
                    'chord_data should be a list of tuples, '
                    + 'but is a list of something else'
                )
            jam.annotations.append(chords_to_jams(chords))

    # keys
    if key_data is not None:
        if type(key_data) != list:
            raise TypeError('key_data should be a list of tuples')
        for keys in key_data:
            if type(keys) != tuple:
                raise TypeError(
                    'key_data should be a list of tuples, '
                    + 'but is a list of something else'
                )
            jam.annotations.append(keys_to_jams(keys))

    # f0
    if f0_data is not None:
        if type(f0_data) != list:
            raise TypeError('f0_data should be a list of tuples')
        for f0s in f0_data:
            if type(f0s) != tuple:
                raise TypeError(
                    'f0_data should be a list of tuples, '
                    + 'but is a list of something else'
                )
            jam.annotations.append(f0s_to_jams(f0s))

    # lyrics
    if lyrics_data is not None:
        if type(lyrics_data) != list:
            raise TypeError('lyrics_data should be a list of tuples')
        for lyrics in lyrics_data:
            if type(lyrics) != tuple:
                raise TypeError(
                    'lyrics_data should be a list of tuples, '
                    + 'but is a list of something else'
                )
            jam.annotations.append(lyrics_to_jams(lyrics))

    return jam


def beats_to_jams(beats):
    '''
    Convert beats annotations into jams format.

    Parameters
    ----------
    beats: tuple
        A tuple in the format (BeatData, str), where str describes the annotation
        and BeatData is the beats mirdata annotation format.

    Returns
    -------
    jannot_beat: JAM beat annotation object.

    '''
    jannot_beat = jams.Annotation(namespace='beat')
    jannot_beat.annotation_metadata = jams.AnnotationMetadata(data_source='mirdata')
    if beats[0] is not None:
        if type(beats[0]) != utils.BeatData:
            raise TypeError('Type should be BeatData.')
        for t, p in zip(beats[0].beat_times, beats[0].beat_positions):
            jannot_beat.append(time=t, duration=0.0, value=p)
    if beats[1] is not None:
        jannot_beat.sandbox = jams.Sandbox(name=beats[1])
    return jannot_beat


def sections_to_jams(sections):
    '''
    Convert sections annotations into jams format.

    Parameters
    ----------
    sections: tuple
        A tuple in the format (SectionData, str), where str describes the annotation
        and SectionData is the sections mirdata annotation format.

    Returns
    -------
    jannot_seg: JAM segment_open annotation object.
    '''
    jannot_seg = jams.Annotation(namespace='segment_open')
    jannot_seg.annotation_metadata = jams.AnnotationMetadata(data_source='mirdata')
    if sections[0] is not None:
        if type(sections[0]) != utils.SectionData:
            raise TypeError('Type should be SectionData.')
        for inter, seg in zip(sections[0].intervals, sections[0].labels):
            jannot_seg.append(time=inter[0], duration=inter[1] - inter[0], value=seg)
    if sections[1] is not None:
        jannot_seg.sandbox = jams.Sandbox(name=sections[1])
    return jannot_seg


def chords_to_jams(chords):
    '''
    Convert chords annotations into jams format.

    Parameters
    ----------
    chords: tuple
        A tuple in the format (ChordData, str), where str describes the annotation
        and ChordData is the chords mirdata annotation format.

    Returns
    -------
    jannot_chord: JAM chord annotation object.
    '''
    jannot_chord = jams.Annotation(namespace='chord')
    jannot_chord.annotation_metadata = jams.AnnotationMetadata(data_source='mirdata')
    if chords[0] is not None:
        if chords[0] is not None:
            if type(chords[0]) != utils.ChordData:
                raise TypeError('Type should be ChordData.')
            for beg, end, ch in zip(
                chords[0].intervals[:, 0], chords[0].intervals[:, 1], chords[0].labels
            ):
                jannot_chord.append(time=beg, duration=end - beg, value=ch)
    if chords[1] is not None:
        jannot_chord.sandbox = jams.Sandbox(name=chords[1])
    return jannot_chord


def keys_to_jams(keys):
    '''
    Convert keys annotations into jams format.

    Parameters
    ----------
    keys: tuple
        A tuple in the format (KeyData, str), where str describes the annotation
        and KeyData is the keys mirdata annotation format.

    Returns
    -------
    jannot_key: JAM key_mode annotation object.
    '''
    jannot_key = jams.Annotation(namespace='key_mode')
    jannot_key.annotation_metadata = jams.AnnotationMetadata(data_source='mirdata')
    if keys[0] is not None:
        if type(keys[0]) != utils.KeyData:
            raise TypeError('Type should be KeyData.')
        for beg, end, key in zip(keys[0].start_times, keys[0].end_times, keys[0].keys):
            jannot_key.append(time=beg, duration=end - beg, value=key)
    if keys[1] is not None:
        jannot_key.sandbox = jams.Sandbox(name=keys[1])
    return jannot_key


def multi_sections_to_jams(multi_sections):
    '''
    Convert hierarchical annotations into jams format.

    Parameters
    ----------
    multi_segment: list
        A list of tuples in the format ([(segments0, level0), (segments1, level1)], annotator),
        where segments are SectionData mirdata format, level indicates the hierarchy (e.g. 0, 1)
        and annotator describes the annotator. This format is customize for Salami dataset annotations.

    Returns
    -------
    jannot_multi: JAM multi_segment annotation object.
    '''
    # sections with multiple annotators and multiple level annotations
    jannot_multi = jams.Annotation(namespace='multi_segment')
    jannot_multi.annotation_metadata = jams.AnnotationMetadata(data_source='mirdata')
    jannot_multi.annotation_metadata = jams.AnnotationMetadata(
        annotator={'name': multi_sections[1]}
    )
    for sections in multi_sections[0]:
        if sections[0] is not None:
            if type(sections[0]) != utils.SectionData:
                raise TypeError('Type should be SectionData.')
            for inter, seg in zip(sections[0].intervals, sections[0].labels):
                jannot_multi.append(
                    time=inter[0],
                    duration=inter[1] - inter[0],
                    value={'label': seg, 'level': sections[1]},
                )
    return jannot_multi


def f0s_to_jams(f0s):
    '''
    Convert f0 annotations into jams format.

    Parameters
    ----------
    f0s: tuple
        A tuple in the format (F0Data, str), where str describes the annotation
        and F0Data is the f0 mirdata annotation format.

    Returns
    -------
    jannot_f0: JAM pitch_contour annotation object.
    '''
    jannot_f0 = jams.Annotation(namespace='pitch_contour')
    jannot_f0.annotation_metadata = jams.AnnotationMetadata(data_source='mirdata')
    if f0s[0] is not None:
        if type(f0s[0]) != utils.F0Data:
            raise TypeError('Type should be F0Data.')
        for t, f, c in zip(f0s[0].times, f0s[0].frequencies, f0s[0].confidence):
            jannot_f0.append(time=t, duration=0.0, value=f, confidence=c)
    if f0s[1] is not None:
        jannot_f0.sandbox = jams.Sandbox(name=f0s[1])
    return jannot_f0


def lyrics_to_jams(lyrics):
    '''
    Convert lyrics annotations into jams format.

    Parameters
    ----------
    lyrics: tuple
        A tuple in the format (LyricData, str), where str describes the annotation
        and LyricData is the lyric mirdata annotation format.

    Returns
    -------
    jannot_lyric: JAM lyric annotation object.
    '''
    jannot_lyric = jams.Annotation(namespace='lyrics')
    jannot_lyric.annotation_metadata = jams.AnnotationMetadata(data_source='mirdata')
    if lyrics[0] is not None:
        if type(lyrics[0]) != utils.LyricData:
            raise TypeError('Type should be LyricData.')
        for beg, end, lyric in zip(
            lyrics[0].start_times, lyrics[0].end_times, lyrics[0].lyrics
        ):
            jannot_lyric.append(time=beg, duration=end - beg, value=lyric)
    if lyrics[1] is not None:
        jannot_lyric.sandbox = jams.Sandbox(name=lyrics[1])
    return jannot_lyric



# -*- coding: utf-8 -*-
from __future__ import absolute_import

import numpy as np
import os

import pytest

from mirdata import tinysol, utils
from tests.test_utils import DEFAULT_DATA_HOME
from tests.test_download_utils import mock_downloader


def test_track():
    # test data home None
    track_default = tinysol.Track('Fl-ord-C4-mf-N-T14d')
    assert track_default._data_home == os.path.join(DEFAULT_DATA_HOME, 'TinySOL')

    # test with custom data_home
    data_home = 'tests/resources/mir_datasets/TinySOL'

    # missing track
    with pytest.raises(ValueError):
        tinysol.Track('asdfasdf', data_home=data_home)

    # test with a wind instrument
    track = tinysol.Track('Fl-ord-C4-mf-N-T14d', data_home=data_home)
    assert track.track_id == 'Fl-ord-C4-mf-N-T14d'
    assert track._data_home == data_home
    y, sr = track.audio
    assert y.shape == (136209,)
    assert sr == 22050
    repr_string = 'TinySOL Track(instrument=Flute, pitch=C4, dynamics=mf)'
    assert track.__repr__() == repr_string

    # test with a string instrument
    track = tinysol.Track('Cb-ord-A2-mf-2c-N', data_home=data_home)
    repr_string = (
        'TinySOL Track(instrument=Contrabass, pitch=A2, ' + 'dynamics=mf, string=II)'
    )
    assert track.__repr__() == repr_string


def test_to_jams():
    data_home = 'tests/resources/mir_datasets/TinySOL'

    # Case with a wind instrument (no string_id)
    track = tinysol.Track('Fl-ord-C4-mf-N-T14d', data_home=data_home)
    jam = track.to_jams()

    assert jam['sandbox']['Fold'] == 0
    assert jam['sandbox']['Family'] == 'Winds'
    assert jam['sandbox']['Instrument (abbr.)'] == 'Fl'
    assert jam['sandbox']['Instrument (in full)'] == 'Flute'
    assert jam['sandbox']['Technique (abbr.)'] == 'ord'
    assert jam['sandbox']['Technique (in full)'] == 'ordinario'
    assert jam['sandbox']['Pitch'] == 'C4'
    assert jam['sandbox']['Pitch ID'] == 60
    assert jam['sandbox']['Dynamics'] == 'mf'
    assert jam['sandbox']['Dynamics ID'] == 2
    assert jam['sandbox']['Instance ID'] == 0
    assert 'String ID' not in jam['sandbox']
    assert jam['sandbox']['Resampled']

    # Case with a string instrument
    track = tinysol.Track('Cb-ord-A2-mf-2c-N', data_home=data_home)
    jam = track.to_jams()

    assert jam['sandbox']['Fold'] == 4
    assert jam['sandbox']['Family'] == 'Strings'
    assert jam['sandbox']['Instrument (abbr.)'] == 'Cb'
    assert jam['sandbox']['Instrument (in full)'] == 'Contrabass'
    assert jam['sandbox']['Technique (abbr.)'] == 'ord'
    assert jam['sandbox']['Technique (in full)'] == 'ordinario'
    assert jam['sandbox']['Pitch'] == 'A2'
    assert jam['sandbox']['Pitch ID'] == 45
    assert jam['sandbox']['Dynamics'] == 'mf'
    assert jam['sandbox']['Dynamics ID'] == 2
    assert jam['sandbox']['Instance ID'] == 1
    assert jam['sandbox']['String ID'] == 2
    assert not jam['sandbox']['Resampled']


def test_track_ids():
    track_ids = tinysol.track_ids()
    assert type(track_ids) is list
    assert len(track_ids) == 2913


def test_load():
    data_home = 'tests/resources/mir_datasets/TinySOL'
    tinysol_data = tinysol.load(data_home=data_home)
    assert type(tinysol_data) is dict
    assert len(tinysol_data.keys()) == 2913

    tinysol_data = tinysol.load()
    assert type(tinysol_data) is dict
    assert len(tinysol_data.keys()) == 2913


def test_download(mock_downloader):
    tinysol.download()
    mock_downloader.assert_called()


def test_validate():
    tinysol.validate()
    tinysol.validate(silence=True)


def test_cite():
    cite_str = tinysol.cite()



# -*- coding: utf-8 -*-
"""DALI Dataset Loader

DALI Dataset.

Details can be found at https://github.com/gabolsgabs/DALI

Attributes:
    DATASET_DIR (str): The directory name for DALI dataset. Set to `'DALI'`.

    DATA.index (dict): {track_id: track_data}.
        track_data is a jason data loaded from `index/`

    DATA.metadata (dict): #TODO


"""

from __future__ import absolute_import
from __future__ import division
from __future__ import print_function

import json
import gzip
import pickle
import os
import librosa
import logging
import numpy as np

# this is the package, needed to load the annotations.
# DALI-dataset is only installed if the user explicitly declares
# they want dali when pip installing.
try:
    import DALI
except ImportError as E:
    logging.error(
        'In order to use dali you must have dali-dataset installed. '
        'Please reinstall mirdata using `pip install \'mirdata[dali]\''
    )
    raise

import mirdata.utils as utils

DATASET_DIR = 'DALI'


def _load_metadata(data_home):
    metadata_path = os.path.join(data_home, os.path.join('dali_metadata.json'))
    if not os.path.exists(metadata_path):
        logging.info('Metadata file {} not found.'.format(metadata_path))
        return None
    with open(metadata_path, 'r') as fhandle:
        metadata_index = json.load(fhandle)

    metadata_index['data_home'] = data_home
    return metadata_index


DATA = utils.LargeData('dali_index.json', _load_metadata)


class Track(object):
    """DALI melody Track class

    Args:
        track_id (str): track id of the track
        data_home (str): Local path where the dataset is stored.
            If `None`, looks for the data in the default directory, `~/mir_datasets`

    Attributes:
        track_id (str): track id
        audio_path (str): track audio path
        artist
        title
        dataset_version (float): dali version
        ground-truth (bool): if it is part of ground-truth or not
        scorea: {'NCC': , 'manual'},
        audio_metadata: {'url': , 'working': },
        album:,
        release_date:,
        language:,
        audio:,
    """

    def __init__(self, track_id, data_home=None):
        if track_id not in DATA.index:
            raise ValueError('{} is not a valid track ID in DALI'.format(track_id))

        if data_home is None:
            data_home = utils.get_default_dataset_path(DATASET_DIR)

        self.track_id = track_id
        self._data_home = data_home
        self._track_paths = DATA.index[track_id]

        metadata = DATA.metadata(data_home)
        if metadata is not None and track_id in metadata:
            self._track_metadata = metadata[track_id]
            self._track_metadata['album'] = metadata[track_id]['metadata']['album']
            self._track_metadata['release_date'] = metadata[track_id]['metadata'][
                'release_date'
            ]
            self._track_metadata['language'] = metadata[track_id]['metadata'][
                'language'
            ]
            self.audio_url = self._track_metadata['audio']['url']
            self.url_working = self._track_metadata['audio']['working']
            self.ground_truth = self._track_metadata['ground-truth']
            self.artist = self._track_metadata['artist']
            self.title = self._track_metadata['title']
            self.dataset_version = self._track_metadata['dataset_version']
            self.scores_ncc = self._track_metadata['scores']['NCC']
            self.scores_manual = self._track_metadata['scores']['manual']
            self.album = self._track_metadata['album']
            self.release_date = self._track_metadata['release_date']
            self.language = self._track_metadata['language']
            self.audio_path = os.path.join(
                self._data_home, self._track_paths['audio'][0]
            )

    def __repr__(self):
        repr_string = (
            "DALI Track(track_id={}, audio_path={}, "
            + "audio_url={}, audio_working={}, ground_truth={}, artist={}, title={},"
            + "dataset_version={}, scores_ncc={}, scores_manual={}, album={}, release_date={}, language={})"
        )
        return repr_string.format(
            self.track_id,
            self.audio_path,
            self.audio_url,
            self.url_working,
            self.ground_truth,
            self.artist,
            self.title,
            self.dataset_version,
            round(self.scores_ncc, 4),
            self.scores_manual,
            self.album,
            self.release_date,
            self.language,
        )

    @utils.cached_property
    def notes(self):
        return _load_annotations_granularity(
            os.path.join(self._data_home, self._track_paths['annot'][0]), 'notes'
        )

    @utils.cached_property
    def words(self):
        return _load_annotations_granularity(
            os.path.join(self._data_home, self._track_paths['annot'][0]), 'words'
        )

    @utils.cached_property
    def lines(self):
        return _load_annotations_granularity(
            os.path.join(self._data_home, self._track_paths['annot'][0]), 'lines'
        )

    @utils.cached_property
    def paragraphs(self):
        return _load_annotations_granularity(
            os.path.join(self._data_home, self._track_paths['annot'][0]), 'paragraphs'
        )

    @utils.cached_property
    def annotation_object(self):
        return _load_annotations_class(
            os.path.join(self._data_home, self._track_paths['annot'][0])
        )

    @property
    def audio(self):
        return librosa.load(self.audio_path, sr=None, mono=True)


def download(data_home=None):
    """DALI is not available for downloading directly.
    This function prints a helper message to download DALI
    through zenodo.org.

    Args:
        data_home (str): Local path where the dataset is stored.
            If `None`, looks for the data in the default directory, `~/mir_datasets`
    """

    if data_home is None:
        data_home = utils.get_default_dataset_path(DATASET_DIR)

    print(
        """
        To download this dataset, visit:
        https://zenodo.org/record/2577915 and request access.

        Once downloaded, unzip the file DALI_v1.0.zip
        and place the result in:
        {save_path}

        Use the function dali_code.get_audio you can find at:
        https://github.com/gabolsgabs/DALI for getting the audio and place them at:
        {audio_path}

    """.format(
            save_path=os.path.join(data_home, DATASET_DIR, 'annotatios'),
            audio_path=os.path.join(data_home, DATASET_DIR, 'audio'),
        )
    )


def validate(data_home=None, silence=False):
    """Validate if the stored dataset is a valid version

    Args:
        data_home (str): Local path where the dataset is stored.
            If `None`, looks for the data in the default directory, `~/mir_datasets`

    Returns:
        missing_files (list): List of file paths that are in the dataset index
            but missing locally
        invalid_checksums (list): List of file paths that file exists in the dataset
            index but has a different checksum compare to the reference checksum

    """
    if data_home is None:
        data_home = utils.get_default_dataset_path(DATASET_DIR)

    missing_files, invalid_checksums = utils.validator(
        DATA.index, data_home, silence=silence
    )
    return missing_files, invalid_checksums


def track_ids():
    """Return track ids

    Returns:
        (list): A list of track ids
    """
    return list(DATA.index.keys())


def load(data_home=None):
    """Load DALI dataset

    Args:
        data_home (str): Local path where the dataset is stored.
            If `None`, looks for the data in the default directory, `~/mir_datasets`

    Returns:
        (dict): {`track_id`: track data}

    """
    if data_home is None:
        data_home = utils.get_default_dataset_path(DATASET_DIR)

    dali_data = {}
    for key in track_ids():
        dali_data[key] = Track(key, data_home=data_home)
    return dali_data


def _load_annotations_granularity(annotations_path, granularity):
    if not os.path.exists(annotations_path):
        return None
    try:
        with gzip.open(annotations_path, 'rb') as f:
            output = pickle.load(f)
    except Exception as e:
        with gzip.open(annotations_path, 'r') as f:
            output = pickle.load(f)
    text = []
    notes = []
    begs = []
    ends = []
    for annot in output.annotations['annot'][granularity]:
        notes.append(round(annot['freq'][0], 3))
        begs.append(round(annot['time'][0], 3))
        ends.append(round(annot['time'][1], 3))
        text.append(annot['text'])
    if granularity == 'notes':
        annotation = utils.NoteData(
            np.array(begs), np.array(ends), np.array(notes), None
        )
    else:
        annotation = utils.LyricData(
            np.array(begs), np.array(ends), np.array(text), None
        )
    return annotation


def _load_annotations_class(annotations_path):
    if not os.path.exists(annotations_path):
        return None
    try:
        with gzip.open(annotations_path, 'rb') as f:
            output = pickle.load(f)
    except Exception as e:
        with gzip.open(annotations_path, 'r') as f:
            output = pickle.load(f)
    return output


def cite():
    """Print the reference"""

    cite_data = """
    ===========  MLA ===========
    Meseguer-Brocal, Gabriel, et al.
    "DALI: a large Dataset of synchronized Audio, LyrIcs and notes, automatically created using teacher-student machine
    learning paradigm."
    In Proceedings of the 19th International Society for Music Information Retrieval Conference (ISMIR). 2018.

    ========== Bibtex ==========
    @inproceedings{Meseguer-Brocal_2018,
    Title = {DALI: a large Dataset of synchronized Audio, LyrIcs and notes, automatically created using teacher-student
     machine learning paradigm.},
    Author = {Meseguer-Brocal, Gabriel and Cohen-Hadria, Alice and Peeters, Geoffroy},
    Booktitle = {19th International Society for Music Information Retrieval Conference},
    Editor = {ISMIR}, Month = {September},
    Year = {2018}}
    """
    print(cite_data)



# -*- coding: utf-8 -*-

from __future__ import absolute_import
from __future__ import division
from __future__ import print_function

import os

import pytest

from mirdata import gtzan_genre
from tests.test_utils import DEFAULT_DATA_HOME
from tests.test_download_utils import mock_downloader

TEST_DATA_HOME = "tests/resources/mir_datasets/GTZAN-Genre"


def test_track_default_data_home():
    # test data home None
    track_default = gtzan_genre.Track("country.00000")
    assert track_default._data_home == os.path.join(DEFAULT_DATA_HOME, "GTZAN-Genre")


def test_unknown_track():
    with pytest.raises(ValueError):
        gtzan_genre.Track("asdfasdf", data_home=TEST_DATA_HOME)


def test_load_track():
    track = gtzan_genre.Track("country.00000", data_home=TEST_DATA_HOME)
    assert track.genre == "country"
    assert track.audio_path == os.path.join(
        TEST_DATA_HOME, "gtzan_genre/genres", "country/country.00000.wav"
    )
    assert track.audio()[0].shape == (663300,)


def test_load():
    data = gtzan_genre.load(data_home=TEST_DATA_HOME)
    assert len(data) == 1000
    key, track = list(sorted(data.items()))[0]
    assert key == "blues.00000"
    assert track.genre == "blues"


def test_validate():
    missing_files, invalid_checksums = gtzan_genre.validate(
        data_home=TEST_DATA_HOME, silence=True
    )

    assert len(missing_files) == 999
    assert len(invalid_checksums) == 0


def test_download(mock_downloader):
    gtzan_genre.download()
    mock_downloader.assert_called()


def test_repr():
    track = gtzan_genre.Track("country.00000", data_home=TEST_DATA_HOME)
    assert str(track) == "GTZAN-Genre Track(track_id='country.00000', genre='country')"


def test_cite(capsys):
    gtzan_genre.cite()
    captured = capsys.readouterr()
    assert "Tzanetakis, George" in captured.out



""" Setup script for mir_datasets. """
from importlib.machinery import SourceFileLoader
from setuptools import setup, find_packages

version_sfl = SourceFileLoader('mirdata.version', 'mirdata/version.py')
version_module = version_sfl.load_module()

with open('README.md', 'r') as fdesc:
    long_description = fdesc.read()

if __name__ == '__main__':
    setup(
        name='mirdata',
        version=version_module.version,
        description='Common loaders for MIR datasets.',
        url='https://github.com/mir-dataset-loaders/mirdata',
        packages=find_packages(exclude=['test', '*.test', '*.test.*']),
        download_url='http://github.com/mir-dataset-loaders/mirdata/releases',
        package_data={'mirdata': ['indexes/*']},
        long_description=long_description,
        long_description_content_type='text/markdown',
        classifiers=[
            "License :: OSI Approved :: BSD License",
            "Programming Language :: Python",
            "Development Status :: 3 - Alpha",
            "Intended Audience :: Science/Research",
            "Topic :: Multimedia :: Sound/Audio :: Analysis",
            "Programming Language :: Python :: 3.6",
            "Programming Language :: Python :: 3.7",
        ],
        keywords='mir dataset loader audio',
        license='BSD-3-Clause',
        install_requires=[
            'tqdm',
            'librosa >= 0.7.0',
            'numpy>=1.16',
            'six',
            'jams',
            'requests',
        ],
        extras_require={
            'tests': [
                'pytest>=4.4.0',
                'pytest-cov>=2.6.1',
                'pytest-pep8>=1.0.0',
                'pytest-mock>=1.10.1',
                'pytest-localserver>=0.5.0',
                'testcontainers>=2.3',
                'future==0.17.1',
                'coveralls>=1.7.0',
                'dali-dataset',
            ],
            'docs': [
                'sphinx',
                'sphinxcontrib-napoleon',
                'sphinx_rtd_theme',
                'numpydoc',
            ],
            'dali': ['DALI-dataset>=1.0.0'],
        },
    )



# -*- coding: utf-8 -*-
"""TinySOL Dataset Loader.

TinySOL is a dataset of 2913 samples, each containing a single musical note from one of 14 different instruments:

    Bass Tuba
    French Horn
    Trombone
    Trumpet in C
    Accordion
    Contrabass
    Violin
    Viola
    Violoncello
    Bassoon
    Clarinet in B-flat
    Flute
    Oboe
    Alto Saxophone


These sounds were originally recorded at Ircam in Paris (France) between 1996
and 1999, as part of a larger project named Studio On Line (SOL). Although SOL
contains many combinations of mutes and extended playing techniques, TinySOL
purely consists of sounds played in the so-called "ordinary" style, and in
absence of mute.

TinySOL can be used for education and research purposes. In particular, it can
be employed as a dataset for training and/or evaluating music information
retrieval (MIR) systems, for tasks such as instrument recognition or
fundamental frequency estimation. For this purpose, we provide an official 5-fold
split of TinySOL as a metadata attribute. This split has been carefully balanced
in terms of instrumentation, pitch range, and dynamics. For the sake of research
reproducibility, we encourage users of TinySOL to adopt this split and report
their results in terms of average performance across folds.

We encourage TinySOL users to subscribe to the Ircam Forum so that they can
have access to larger versions of SOL.
For more information, see https://www.orch-idea.org/

Attributes:
    DATA.index (dict): {track_id: track_data}.
        track_id is a JSON data loaded from 'indexes/'

    DATASET_DIR (str): The directory name for TinySOL.
        Set to `'TinySOL'`.

    DATA.metadata (dict): The metadata of TinySOL.
"""

from __future__ import absolute_import
from __future__ import division
from __future__ import print_function

import csv
import jams
import librosa
import logging
import os

import mirdata.utils as utils
import mirdata.download_utils as download_utils
import mirdata.jams_utils as jams_utils

DATASET_DIR = "TinySOL"
AUDIO_REMOTE = download_utils.RemoteFileMetadata(
    filename="TinySOL.tar.gz",
    url="https://zenodo.org/record/3685367/files/TinySOL.tar.gz?download=1",
    checksum="36030a7fe389da86c3419e5ee48e3b7f",
    destination_dir="audio",
)
ANNOTATION_REMOTE = download_utils.RemoteFileMetadata(
    filename="TinySOL_metadata.csv",
    url="https://zenodo.org/record/3685367/files/TinySOL_metadata.csv?download=1",
    checksum="a86c9bb115f69e61f2f25872e397fc4a",
    destination_dir="annotation",
)
STRING_ROMAN_NUMERALS = {1: "I", 2: "II", 3: "III", 4: "IV"}


def _load_metadata(data_home):
    metadata_path = os.path.join(data_home, "annotation", "TinySOL_metadata.csv")

    if not os.path.exists(metadata_path):
        logging.info("Metadata file {} not found.".format(metadata_path))
        return None

    metadata_index = {}
    with open(metadata_path, "r") as fhandle:
        csv_reader = csv.reader(fhandle, delimiter=",")
        next(csv_reader)
        for row in csv_reader:
            key = os.path.splitext(os.path.split(row[0])[1])[0]
            metadata_index[key] = {
                "Fold": int(row[1]),
                "Family": row[2],
                "Instrument (abbr.)": row[3],
                "Instrument (in full)": row[4],
                "Technique (abbr.)": row[5],
                "Technique (in full)": row[6],
                "Pitch": row[7],
                "Pitch ID": int(row[8]),
                "Dynamics": row[9],
                "Dynamics ID": int(row[10]),
                "Instance ID": int(row[11]),
                "Resampled": (row[13] == "TRUE"),
            }
            if len(row[12]) > 0:
                metadata_index[key]["String ID"] = int(float(row[12]))

    metadata_index["data_home"] = data_home

    return metadata_index


DATA = utils.LargeData("tinysol_index.json", _load_metadata)


class Track(object):
    """TinySOL track class

    Args:
        track_id (str): ID of the track
        data_home (str): Local path where the dataset is stored.
            If `None`, looks for the data in the default directory, `~/mir_datasets`

    Attributes:
        family (str): instrument family encoded by its English name
        instrument_abbr (str): instrument abbreviation
        instrument_full (str): instrument encoded by its English name
        technique_abbr (str): playing technique abbreviation
        technique_full (str): playing technique encoded by its English name
        pitch (str): string containing English pitch class and octave number
        pitch_id (int): MIDI note index, where middle C ("C4") corresponds to 60
        dynamics (str): dynamics abbreviation. Ex: pp, mf, ff, etc.
        dynamics_id (int): pp=0, p=1, mf=2, f=3, ff=4
        is_resampled (bool): True if this sample was pitch-shifted from a neighbor; False if it was genuinely recorded.
        instance_id (int): instance ID. Either equal to 0, 1, 2, or 3.
        string_id (int or None): string ID. By musical convention, the first
        string is the highest. On wind instruments, this is replaced by `None`.
    """

    def __init__(self, track_id, data_home=None):
        if track_id not in DATA.index:
            raise ValueError("{} is not a valid track ID in TinySOL".format(track_id))

        self.track_id = track_id

        if data_home is None:
            data_home = utils.get_default_dataset_path(DATASET_DIR)

        self._data_home = data_home
        self._track_paths = DATA.index[track_id]

        metadata = DATA.metadata(data_home)
        if metadata is not None and track_id in metadata:
            self._track_metadata = metadata[track_id]
        else:
            self._track_metadata = {
                "Family": None,
                "Instrument (abbr.)": None,
                "Instrument (in full)": None,
                "Technique (abbr.)": None,
                "Technique (in full)": None,
                "Pitch": None,
                "Pitch ID": None,
                "Dynamics": None,
                "Dynamics ID": None,
                "Instance ID": None,
                "String ID": None,
                "Resampled": None,
            }

        self.audio_path = os.path.join(
            self._data_home, "audio", self._track_paths["audio"][0]
        )

        self.family = self._track_metadata["Family"]
        self.instrument_abbr = self._track_metadata["Instrument (abbr.)"]
        self.instrument_full = self._track_metadata["Instrument (in full)"]
        self.technique_abbr = self._track_metadata["Technique (abbr.)"]
        self.technique_full = self._track_metadata["Technique (in full)"]
        self.pitch = self._track_metadata["Pitch"]
        self.pitch_id = self._track_metadata["Pitch ID"]
        self.dynamics = self._track_metadata["Dynamics"]
        self.dynamics_id = self._track_metadata["Dynamics ID"]
        self.instance_id = self._track_metadata["Instance ID"]
        if "String ID" in self._track_metadata:
            self.string_id = self._track_metadata["String ID"]
        else:
            self.string_id = None
        self.is_resampled = self._track_metadata["Resampled"]

    def __repr__(self):

        if self.string_id:
            repr_string = (
                "TinySOL Track(instrument={}, pitch={}, dynamics={}, string={})"
            ).format(
                self.instrument_full,
                self.pitch,
                self.dynamics,
                STRING_ROMAN_NUMERALS[self.string_id],
            )
        else:
            repr_string = (
                "TinySOL Track(instrument={}, pitch={}, dynamics={})"
            ).format(self.instrument_full, self.pitch, self.dynamics)
        return repr_string

    @property
    def audio(self):
        return librosa.load(self.audio_path, sr=22050, mono=True)

    def to_jams(self):
        return jams_utils.jams_converter(
            metadata=self._track_metadata
        )  # TODO pass a note_data?


def download(data_home=None):
    """Download TinySOL.

    Args:
        data_home (str): Local path where the dataset is stored.
            If `None`, looks for the data in the default directory, `~/mir_datasets`
    """
    if data_home is None:
        data_home = utils.get_default_dataset_path(DATASET_DIR)

    download_utils.downloader(
        data_home,
        tar_downloads=[AUDIO_REMOTE],
        file_downloads=[ANNOTATION_REMOTE],
        cleanup=True,
    )


def track_ids():
    """Return track ids

    Returns:
        (list): A list of track ids
    """
    return list(DATA.index.keys())


def validate(data_home=None, silence=False):
    """Validate if the stored dataset is a valid version

    Args:
        data_home (str): Local path where the dataset is stored.
            If `None`, looks for the data in the default directory, `~/mir_datasets`

    Returns:
        missing_files (list): List of file paths that are in the dataset index
            but missing locally
        invalid_checksums (list): List of file paths that file exists in the dataset
            index but has a different checksum compare to the reference checksum
    """
    if data_home is None:
        data_home = utils.get_default_dataset_path(DATASET_DIR)

    missing_files, invalid_checksums = utils.validator(
        DATA.index, os.path.join(data_home, "audio"), silence=silence
    )
    return missing_files, invalid_checksums


def load(data_home=None):
    """Load TinySOL
    Args:
        data_home (str): Local path where TinySOL is stored.
            If `None`, looks for the data in the default directory, `~/mir_datasets`
    Returns:
        (dict): {`track_id`: track data}
    """
    if data_home is None:
        data_home = utils.get_default_dataset_path(DATASET_DIR)

    tinysol_data = {}
    for key in DATA.index.keys():
        tinysol_data[key] = Track(key, data_home=data_home)
    return tinysol_data


def cite():
    """Print the reference"""

    cite_data = """
=========== MLA ===========
Cella, Carmine Emanuele, et al., "OrchideaSOL: A dataset of extended
instrumental techniques for computer-aided orchestration". Under review, 2020.

========== Bibtex ==========
@inproceedings{cella2020preprint,
author={Cella, Carmine Emanuele and Ghisi, Daniele and Lostanlen, Vincent and
Lévy, Fabien and Fineberg, Joshua and Maresz, Yan},
title={{OrchideaSOL}: {A} dataset of extended
instrumental techniques for computer-aided orchestration},
bootktitle={Under review},
year={2020}}
"""
    print(cite_data)



from __future__ import absolute_import

import os

import numpy as np
import pytest
import DALI

from mirdata import dali, utils
from tests.test_utils import DEFAULT_DATA_HOME


def test_track():
    # test data home None
    track_default = dali.Track('4b196e6c99574dd49ad00d56e132712b')
    assert track_default._data_home == os.path.join(DEFAULT_DATA_HOME, 'DALI')

    data_home = 'tests/resources/mir_datasets/DALI'

    with pytest.raises(ValueError):
        dali.Track('asdf', data_home=data_home)

    track = dali.Track('4b196e6c99574dd49ad00d56e132712b', data_home=data_home)
    assert track.track_id == '4b196e6c99574dd49ad00d56e132712b'
    assert track._data_home == data_home
    assert track._track_paths == {
        'audio': [
            'audio/4b196e6c99574dd49ad00d56e132712b.mp3',
            '5f01ab8cd5efe947b1a2944e78e55258',
        ],
        'annot': [
            'annotations/4b196e6c99574dd49ad00d56e132712b.gz',
            'c99a5ce0b1581f2420d0706c6f7f7118',
        ],
    }
    assert (
        track.audio_path
        == 'tests/resources/mir_datasets/DALI/'
        + 'audio/4b196e6c99574dd49ad00d56e132712b.mp3'
    )
    assert track.title == 'B.Y.O.B.'
    assert type(track.notes) == utils.NoteData
    assert type(track.words) == utils.LyricData
    assert type(track.paragraphs) == utils.LyricData
    assert type(track.annotation_object) == DALI.Annotations

    path_save = '/home/mfuentes/astre/code/repositories/mirdata/tests/resources/mir_datasets/DALI/annotations'
    name = 'test'
    track.annotation_object.write_json(path_save, name)

    audio, sr = track.audio
    assert sr == 48000
    assert audio.shape == (94208,)

    repr_string = (
        "DALI Track(track_id=4b196e6c99574dd49ad00d56e132712b, "
        + "audio_path=tests/resources/mir_datasets/DALI/audio/4b196e6c99574dd49ad00d56e132712b.mp3, "
        + "audio_url=zUzd9KyIDrM, audio_working=True, ground_truth=False, artist=System Of A Down, "
        + "title=B.Y.O.B.,dataset_version=1, scores_ncc=0.9645, scores_manual=0, "
        + "album=Mezmerize, release_date=2005, language=english)"
    )
    assert track.__repr__() == repr_string


def test_track_ids():
    track_ids = dali.track_ids()
    assert type(track_ids) is list
    assert len(track_ids) == 5358


def test_load():
    data_home = 'tests/resources/mir_datasets/DALI'
    dali_data = dali.load(data_home=data_home)
    assert type(dali_data) is dict
    assert len(dali_data.keys()) == 5358

    dali_data_default = dali.load()
    assert type(dali_data_default) is dict
    assert len(dali_data_default.keys()) == 5358


def test_load_notes():
    notes_path = (
        'tests/resources/mir_datasets/DALI/annotations/'
        + '4b196e6c99574dd49ad00d56e132712b.gz'
    )
    note_data = dali._load_annotations_granularity(notes_path, 'notes')

    assert type(note_data) == utils.NoteData
    assert type(note_data.start_times) == np.ndarray
    assert type(note_data.end_times) == np.ndarray
    assert type(note_data.notes) == np.ndarray

    assert np.array_equal(note_data.start_times, np.array([24.125, 24.273, 24.420]))
    assert np.array_equal(note_data.end_times, np.array([24.273, 24.420, 24.568]))
    assert np.array_equal(note_data.notes, np.array([1108.731, 1108.731, 1108.731]))

    # load a file which doesn't exist
    notes_none = dali._load_annotations_granularity('fake/file/path', 'notes')
    assert notes_none is None


def test_load_words():
    data_path = (
        'tests/resources/mir_datasets/DALI/annotations/'
        + '4b196e6c99574dd49ad00d56e132712b.gz'
    )
    word_data = dali._load_annotations_granularity(data_path, 'words')

    assert type(word_data) == utils.LyricData
    assert type(word_data.start_times) == np.ndarray
    assert type(word_data.end_times) == np.ndarray
    assert type(word_data.lyrics) == np.ndarray

    assert np.array_equal(word_data.start_times, np.array([24.125, 24.273, 24.42]))
    assert np.array_equal(word_data.end_times, np.array([24.273, 24.42, 24.568]))
    assert np.array_equal(word_data.lyrics, np.array(['why', 'do', 'they']))

    # load a file which doesn't exist
    words_none = dali._load_annotations_granularity('fake/file/path', 'words')
    assert words_none is None


def test_load_lines():
    data_path = (
        'tests/resources/mir_datasets/DALI/annotations/'
        + '4b196e6c99574dd49ad00d56e132712b.gz'
    )
    line_data = dali._load_annotations_granularity(data_path, 'lines')

    assert type(line_data) == utils.LyricData
    assert type(line_data.start_times) == np.ndarray
    assert type(line_data.end_times) == np.ndarray
    assert type(line_data.lyrics) == np.ndarray

    print(line_data.start_times)
    print(line_data.end_times)
    print(line_data.lyrics)

    assert np.array_equal(line_data.start_times, np.array([24.125, 24.42]))
    assert np.array_equal(line_data.end_times, np.array([24.42, 24.568]))
    assert np.array_equal(line_data.lyrics, np.array(['why do', 'they']))

    # load a file which doesn't exist
    line_none = dali._load_annotations_granularity('fake/file/path', 'lines')
    assert line_none is None


def test_load_paragraphs():
    data_path = (
        'tests/resources/mir_datasets/DALI/annotations/'
        + '4b196e6c99574dd49ad00d56e132712b.gz'
    )
    par_data = dali._load_annotations_granularity(data_path, 'paragraphs')

    assert type(par_data) == utils.LyricData
    assert type(par_data.start_times) == np.ndarray
    assert type(par_data.end_times) == np.ndarray
    assert type(par_data.lyrics) == np.ndarray

    assert np.array_equal(par_data.start_times, np.array([24.125, 24.420]))
    assert np.array_equal(par_data.end_times, np.array([24.420, 24.568]))
    assert np.array_equal(par_data.lyrics, np.array(['why do', 'they']))

    # load a file which doesn't exist
    pars_none = dali._load_annotations_granularity('fake/file/path', 'paragraphs')
    assert pars_none is None


def test_load_dali_object():
    data_path = (
        'tests/resources/mir_datasets/DALI/annotations/'
        + '4b196e6c99574dd49ad00d56e132712b.gz'
    )
    dali_data = dali._load_annotations_class(data_path)

    assert type(dali_data) == DALI.Annotations
    assert dali_data.annotations['annot']['notes'] == [
        {
            'text': 'why',
            'freq': [1108.7305239074883, 1108.7305239074883],
            'time': [24.12471002069169, 24.272507833284063],
            'index': 0,
        },
        {
            'text': 'do',
            'freq': [1108.7305239074883, 1108.7305239074883],
            'time': [24.272507833284063, 24.42030564587644],
            'index': 1,
        },
        {
            'text': 'they',
            'freq': [1108.7305239074883, 1108.7305239074883],
            'time': [24.42030564587644, 24.568103458468812],
            'index': 2,
        },
    ]
    assert dali_data.annotations['annot']['words'] == [
        {
            'text': 'why',
            'freq': [1108.7305239074883, 1108.7305239074883],
            'time': [24.12471002069169, 24.272507833284063],
            'index': 0,
        },
        {
            'text': 'do',
            'freq': [1108.7305239074883, 1108.7305239074883],
            'time': [24.272507833284063, 24.42030564587644],
            'index': 0,
        },
        {
            'text': 'they',
            'freq': [1108.7305239074883, 1108.7305239074883],
            'time': [24.42030564587644, 24.568103458468812],
            'index': 1,
        },
    ]
    assert dali_data.annotations['annot']['lines'] == [
        {
            'text': 'why do',
            'freq': [1108.7305239074883, 1108.7305239074883],
            'time': [24.12471002069169, 24.42030564587644],
            'index': 0,
        },
        {
            'text': 'they',
            'freq': [1108.7305239074883, 1108.7305239074883],
            'time': [24.42030564587644, 24.568103458468812],
            'index': 1,
        },
    ]
    assert dali_data.annotations['annot']['paragraphs'] == [
        {
            'text': 'why do',
            'freq': [1108.7305239074883, 1108.7305239074883],
            'time': [24.12471002069169, 24.42030564587644],
        },
        {
            'text': 'they',
            'freq': [1108.7305239074883, 1108.7305239074883],
            'time': [24.42030564587644, 24.568103458468812],
        },
    ]

    # load a file which doesn't exist
    dali_none = dali._load_annotations_class('fake/file/path')
    assert dali_none is None


def test_validate():
    dali.validate()
    dali.validate(silence=True)


def test_cite():
    dali.cite()
